/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2010, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/

// -----------------------------------------------------------------------------
// util.java
// -----------------------------------------------------------------------------

package gov.lbl.srm.client.api;

import java.io.*;
import java.util.*;

import gov.lbl.srm.StorageResourceManager.*;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.*;

public class util {

    public static int mapStatusCode(TStatusCode code) {
       //900 , connection refused
       //901 , GSI mapping not found
       //902 , general unpreditable exception
       //903 , input error
       //904 , other error, not reached SRM yet.
       if(code == null) return 1000; //SRM returned no status, but it is a failure
       else {
          String str = code.toString();
          return mapStatusCode(str);
       }
     }

     public static int mapStatusCode(String str) {

          if(str.equals("SRM_SUCCESS") || str.equals("")) {
            return 0;
          }
          else if(str.equals("SRM_FAILURE")) {
            return 1001;
          }
          else if(str.equals("SRM_AUTHENTICATION_FAILURE")) {
            return 1002;
          }
          else if(str.equals("SRM_AUTHORIZATION_FAILURE")) {
            return 1003;
          }
          else if(str.equals("SRM_INVALID_REQUEST")) {
            return 1004;
          }
          else if(str.equals("SRM_INVALID_PATH")) {
            return 1005;
          }
          else if(str.equals("SRM_FILE_LIFETIME_EXPIRED")) {
            return 1006;
          }
          else if(str.equals("SRM_SPACE_LIFETIME_EXPIRED")) {
            return 1007;
          }
          else if(str.equals("SRM_EXCEED_ALLOCATION")) {
            return 1008;
          }
          else if(str.equals("SRM_NO_USER_SPACE")) {
            return 1009;
          }
          else if(str.equals("SRM_NO_FREE_SPACE")) {
            return 1010;
          }
          else if(str.equals("SRM_DUPLICATION_ERROR")) {
            return 1011;
          }
          else if(str.equals("SRM_NON_EMPTY_DIRECTORY")) {
            return 1012;
          }
          else if(str.equals("SRM_TOO_MANY_RESULTS")) {
            return 1013;
          }
          else if(str.equals("SRM_INTERNAL_ERROR")) {
            return 1014;
          }
          else if(str.equals("SRM_FATAL_INTERNAL_ERROR")) {
            return 1015;
          }
          else if(str.equals("SRM_NOT_SUPPORTED")) {
            return 1016;
          }
          else if(str.equals("SRM_REQUEST_QUEUED")) {
            return 1017;
          }
          else if(str.equals("SRM_REQUEST_INPROGRESS")) {
            return 1018;
          }
          else if(str.equals("SRM_REQUEST_SUSPENDED")) {
            return 1019;
          }
          else if(str.equals("SRM_ABORTED")) {
            return 1020;
          }
          else if(str.equals("SRM_RELEASED")) {
            return 1021;
          }
          else if(str.equals("SRM_FILE_PINNED")) {
            return 1022;
          }
          else if(str.equals("SRM_FILE_IN_CACHE")) {
            return 1023;
          }
          else if(str.equals("SRM_SPACE_AVAILABLE")) {
            return 1024;
          }
          else if(str.equals("SRM_LOWER_SPACE_GRANTED")) {
            return 1025;
          }
          else if(str.equals("SRM_DONE")) {
            return 1026;
          }
          else if(str.equals("SRM_PARTIAL_SUCCESS")) {
            return 1027;
          }
          else if(str.equals("SRM_REQUEST_TIMED_OUT")) {
            return 1028;
          }
          else if(str.equals("SRM_LAST_COPY")) {
            return 1029;
          }
          else if(str.equals("SRM_FILE_BUSY")) {
            return 1030;
          }
          else if(str.equals("SRM_FILE_LOST")) {
            return 1031;
          }
          else if(str.equals("SRM_FILE_UNAVAILABLE")) {
            return 1032;
          }
          //else if(str.equals("SRM_CUSTOM_STATUS"))
          else {
            return 1033;
          }
       }

    public static void printMessage(String msg, boolean silent) { 
       if(silent) {;}
       else {
         System.out.println(msg);
       }
    }

    public static void printEventLog(
            java.util.logging.Logger theLogger, 
			String eventName, Vector inputVec, boolean silent) {

     if(silent) {
       if(theLogger != null) {
         int size = inputVec.size();
         Object [] param = new Object[size];

         for(int i = 0; i < size; i++) {
            param[i] = (String) inputVec.elementAt(i);
         }
         theLogger.log(java.util.logging.Level.FINE, eventName, 
			(Object[]) param);
        }
      }
    }


    public static void printStackTrace(Exception e, java.util.logging.Logger logger) {
      //StackTraceElement[] strace = e.getStackTrace ();
      System.out.println("++++++++++++++++++++++++++++++++++++");
      System.out.println("Exception : " + e.getMessage());
      e.printStackTrace();
      /*
      if(logger != null) {
        Vector inputVec = new Vector ();
        for(int i = 0; i < strace.length; i++) {
           StackTraceElement ss =  strace[i];
           inputVec.addElement("ClassName "  + ss.getClassName());
           inputVec.addElement("FileName "   + ss.getFileName());
           inputVec.addElement("LineNumber " + ss.getLineNumber());
           inputVec.addElement("MethodName " + ss.getMethodName());
           inputVec.addElement("...................................");
        }
        util.printEventLog(logger,"ExceptionStackTrace",inputVec,silent);
      }
      int length = strace.length;
      if(strace.length > 5) length = 5;
      for(int i = 0; i < length ; i++) {
         StackTraceElement ss =  strace[i];
         System.out.println("ClassName "  + ss.getClassName());
         System.out.println("FileName "   + ss.getFileName());
         System.out.println("LineNumber " + ss.getLineNumber());
         System.out.println("MethodName " + ss.getMethodName());
         System.out.println("...................................");
      }
      */
    }

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// parseLocalSourceFileForPath
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
public static String parseLocalSourceFileForPath (String str)
  throws Exception {
 
  if(str.startsWith("file:////")) {
    return str.substring(8);
  }
  else if(str.startsWith("file:///")) {
    return str.substring(7);
  }
  else if(str.startsWith("file://")) {
    return str.substring(6);
  }
  else if(str.startsWith("file:/")) {
    return str.substring(5);
  }
  else if(str.startsWith("/")) {
    return str;
  }
  else {
    throw new Exception("SRM-CLIENT: parseLocalSourceFileForPath:SURL not in correct format " + str);
  }
}


} 
