/**
 *
 * *** Copyright Notice ***
 *
 * BeStMan Copyright (c) 2010, The Regents of the University of California, 
 * through Lawrence Berkeley National Laboratory (subject to receipt of any 
 * required approvals from the U.S. Dept. of Energy).  This software was 
 * developed under funding from the U.S. Department of Energy and is 
 * associated with the Berkeley Lab Scientific Data Management Group projects.
 * All rights reserved.
 * 
 * If you have questions about your rights to use or distribute this software, 
 * please contact Berkeley Lab's Technology Transfer Department at TTD@lbl.gov.
 * 
 * NOTICE.  This software was developed under funding from the 
 * U.S. Department of Energy.  As such, the U.S. Government has been granted 
 * for itself and others acting on its behalf a paid-up, nonexclusive, 
 * irrevocable, worldwide license in the Software to reproduce, prepare 
 * derivative works, and perform publicly and display publicly.  
 * Beginning five (5) years after the date permission to assert copyright is 
 * obtained from the U.S. Department of Energy, and subject to any subsequent 
 * five (5) year renewals, the U.S. Government is granted for itself and others
 * acting on its behalf a paid-up, nonexclusive, irrevocable, worldwide license
 * in the Software to reproduce, prepare derivative works, distribute copies to
 * the public, perform publicly and display publicly, and to permit others to
 * do so.
 *
*/
/**
 *
 * Email questions to SRM@LBL.GOV
 * Scientific Data Management Research Group
 * Lawrence Berkeley National Laboratory
 * http://sdm.lbl.gov/bestman
 *
*/

package gov.lbl.srm.client.wsdl;
 
import gov.lbl.srm.StorageResourceManager.*;
//import gov.lbl.srm.util.*;
import gov.lbl.srm.client.api.*;
import org.globus.axis.util.Util;

import java.io.*;
import java.net.URL;
import java.util.Vector;
import java.util.HashMap;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.StringTokenizer;


import javax.xml.rpc.Stub;
import org.apache.axis.types.URI;
import org.apache.axis.types.UnsignedLong;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.*;
import org.apache.log4j.PropertyConfigurator;

import org.globus.axis.gsi.GSIConstants;
import org.gridforum.jgss.ExtendedGSSCredential;
import org.ietf.jgss.GSSCredential;
import org.gridforum.jgss.ExtendedGSSManager;

//import org.globus.ogsa.utils.GSIUtils;
import org.globus.util.ConfigUtil;
//import org.globus.ogsa.utils.GetOpts;
//import org.globus.ogsa.gui.ServiceContext;
import org.globus.gsi.GlobusCredential;

import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.gsi.gssapi.auth.*;

import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;

import gov.lbl.srm.client.exception.*;

//import org.globus.wsrf.impl.security.authentication.Constants;


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMBringOnlineClient
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

class SRMBringOnlineClient {
  private ISRM _srm;
  private GSSCredential _credential;
  private static boolean _debug;
  private static boolean silent;
  private static java.util.logging.Logger _theLogger;
  private static Vector inputVec = new Vector ();
  private String userDesc = ""; 
  private String authId = "";
  private String  protocolsList ;
  private String spaceToken;
  private int totalRequestTime;
  private int fileLifeTime;
  private int deferredStartTime;
  private String storageInfo;
  private TFileStorageType fileStorageType;
  private TRetentionPolicyInfo retentionPolicyInfo;
  private int bufferSize;
  private boolean dcau;
  private int parallelism;


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//SRMBringOnlineClient --- constructor
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SRMBringOnlineClient (ISRM srm, GSSCredential credential,
	boolean debug, boolean silent, 
	java.util.logging.Logger theLogger, int bufferSize, int parallelism,
	boolean dcau) throws Exception 
{
   _srm = srm;
   _credential = credential;
   _debug = debug;
   this.silent = silent;
   _theLogger = theLogger;
   this.bufferSize = bufferSize;
   this.parallelism = parallelism;
   this.dcau = dcau;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// doBringOnline
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SrmBringOnlineResponse doBringOnline(String[] surls)
	throws SRMClientException, Exception {
   
  try {
   inputVec = new Vector();
   validateInput (surls);
   inputVec.addElement("Validated input");
   util.printEventLog(_theLogger,"doBringOnline",inputVec,silent);
   SrmBringOnlineResponse response = 
		prepareToBringOnlineMultiFiles (surls,spaceToken);
   if(_debug) {
     SRMBringOnlineClient.printBringOnlineResult(response);
   }
   return response;
  }catch(Exception e) {
    inputVec = new Vector ();
    inputVec.addElement("Exception="+e.getMessage());
    util.printEventLog(_theLogger,"doBringOnline",inputVec,silent);
    util.printStackTrace(e,_theLogger);
    throw new SRMClientException(e.getMessage());
  }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// validateInput
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

private void validateInput(String[] surl) throws Exception { 
  for(int i = 0; i < surl.length; i++) {
    if(surl[i].startsWith("srm:")) {; }
    else {
      throw new Exception("Given Input is not valid, SURL must start with srm: "); 
    }
  }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setRetentionPolicyInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
void setRetentionPolicyInfo(TRetentionPolicyInfo retentionPolicy) {
   retentionPolicyInfo = retentionPolicy;
}
 
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setFileStorageInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
void setFileStorageType(TFileStorageType fileStorageType) {
    this.fileStorageType= fileStorageType;
}


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setUserDesc
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setUserDesc(String userDesc) {
  this.userDesc = userDesc;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setAuthID
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setAuthID(String authId) {
  this.authId = authId;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setTargetSpaceToken
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setTargetSpaceToken(String spaceToken) {
  this.spaceToken = spaceToken;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setProtocolsList
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setProtocolsList(String protocolsList) {
  this.protocolsList = protocolsList;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setStorageSystemInfo
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setStorageSystemInfo(String storageInfo) {
   this.storageInfo = storageInfo;
}


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setTotalRequestTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setTotalRequestTime(int totalRequestTime) {
   this.totalRequestTime = totalRequestTime;
}


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//setFileLifeTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setFileLifeTime(int fileLifeTime) {
  this.fileLifeTime = fileLifeTime;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// setDeferredStartTime
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

void setDeferredStartTime(int deferredStartTime) {
  this.deferredStartTime = deferredStartTime;
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// prepareToBringOnlineMultiFiles
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
	
private SrmBringOnlineResponse prepareToBringOnlineMultiFiles(String[] surls,
	String token)  throws Exception {


  /*
  if(token != null) {
      if(_debug)  {
        getSpaceTokenMeta(token);
      }
   }
   */

   ExtendedGSSCredential cred = (ExtendedGSSCredential) _credential;
   byte[] bb = cred.export(ExtendedGSSCredential.IMPEXP_OPAQUE);
   String proxyString = new String(bb);

   SrmBringOnlineRequest r = new SrmBringOnlineRequest();

   inputVec = new Vector();
   inputVec.addElement("UserDescription="+userDesc);
   if(!userDesc.equals("")) {
     if(_debug) {
       util.printMessage("SRM-CLIENT: UserDescription="+userDesc, silent);
     }
     r.setUserRequestDescription(userDesc);
   }

   inputVec.addElement("\tAuthorizationID="+authId);
   if(_debug) {
     util.printMessage("SRM-CLIENT: AuthorizationID="+authId, silent);
   }
   if(!authId.equals("")) {
     r.setAuthorizationID(authId);
   }

   TTransferParameters transferParameters = new TTransferParameters();
   ArrayOfString protocolsArray = new ArrayOfString();
   Vector protocolVec = new Vector ();

   if(protocolsList.equals("")) {
     protocolVec.addElement("gsiftp");
     protocolVec.addElement("http");
     protocolVec.addElement("https");
     protocolVec.addElement("ftp");
   }
   else {
     StringTokenizer st = new StringTokenizer(protocolsList,",");
     while (st.hasMoreTokens()) {
       String vv = st.nextToken(); 
       protocolVec.addElement(vv);
     }
   }


   String [] protocols = new String[protocolVec.size()];
   for(int i = 0; i < protocolVec.size(); i++) {
      protocols[i] = (String) protocolVec.elementAt(i);
   }

   String tt = "";
   for(int i = 0; i < protocols.length;i++) {
     tt = tt+protocols[i]+",";
   }

   inputVec.addElement("Protocols="+tt);
   if(_debug) {
     util.printMessage("SRM-CLIENT: Protocols="+tt, silent);
   }
   protocolsArray.setStringArray(protocols);
   transferParameters.setArrayOfTransferProtocols(protocolsArray);
   r.setTransferParameters(transferParameters);

   if(retentionPolicyInfo != null) {
     if(_debug) {
       util.printMessage("SRM-CLIENT: RetentionPolicy="+retentionPolicyInfo.getRetentionPolicy(),silent);
     } 
     inputVec.addElement("RetentionPolicy="+retentionPolicyInfo.getRetentionPolicy());
     if(_debug) {
       util.printMessage("SRM-CLIENT: ACCESSINFO="+retentionPolicyInfo.getAccessLatency(),silent);
     }
     inputVec.addElement("ACCESSINFO="+retentionPolicyInfo.getAccessLatency());
     r.setTargetFileRetentionPolicyInfo(retentionPolicyInfo);
   }

   if(token != null && !token.equals("")) {
     if(_debug) {
        util.printMessage("SRM-CLIENT: SpaceToken="+token, silent);
     }
     inputVec.addElement("SpaceToken="+token);
     r.setTargetSpaceToken(token);
   }
   if(fileStorageType != null) {
     if(_debug) {
        util.printMessage("SRM-CLIENT: FileStorageType="+fileStorageType, silent);
     }
     inputVec.addElement("FileStorageType="+fileStorageType);
     r.setDesiredFileStorageType(fileStorageType);
   }
   if(totalRequestTime != 0) {
    if(_debug) {
       util.printMessage("SRM-CLIENT: TotalRequestTime="+totalRequestTime, silent);
    }
    inputVec.addElement("TotalRequestTime="+totalRequestTime);
    r.setDesiredTotalRequestTime(new Integer(totalRequestTime));
   }
   if(fileLifeTime != 0) {
    if(_debug) {
       util.printMessage("SRM-CLIENT: LifeTime="+fileLifeTime, silent);
    }
    inputVec.addElement("PinTime="+fileLifeTime);
    r.setDesiredLifeTime(new Integer(fileLifeTime));
   }

   if(deferredStartTime != 0) {
    if(_debug) {
       util.printMessage("SRM-CLIENT: DeferredStartTime="+deferredStartTime, silent);
    }
    inputVec.addElement("DeferredStartTime="+deferredStartTime);
    r.setDeferredStartTime(new Integer(deferredStartTime));
   }

   int total = surls.length;

   TGetFileRequest[] fileReqList = new TGetFileRequest[total];

   for (int i=0; i<total; i++) {
     TGetFileRequest curr = new TGetFileRequest();
     curr.setSourceSURL(new URI(surls[i]));
     fileReqList[i] = curr;

     if(_debug) {
       inputVec.addElement("SURL["+i+"]="+surls[i]);
       util.printMessage("SRM-CLIENT: SURL["+i+"]="+ surls[i],silent);
     }


     if(storageInfo.equals("")) {
       int idx = surls[i].indexOf("mss://");
       if(idx != -1) {
         storageInfo = ""+true;
       }
     }
   }


   ArrayOfTExtraInfo storageInfoArray = new ArrayOfTExtraInfo();
   Vector vec = new Vector();

   TExtraInfo tExtra = new TExtraInfo();
   tExtra.setKey("gsiftpStream");
   tExtra.setValue(""+parallelism);
   vec.addElement(tExtra);
   if(_debug) {
     util.printMessage("SRM-CLIENT: gsiftpStream="+parallelism, silent);
   } 
   inputVec.addElement("gsiftpStream="+parallelism);

   tExtra = new TExtraInfo();
   tExtra.setKey("bufferSize");
   tExtra.setValue(""+bufferSize);
   vec.addElement(tExtra);
   if(_debug) {
      util.printMessage("SRM-CLIENT: BufferSize="+bufferSize, silent);
   }
   inputVec.addElement("BufferSize="+bufferSize);

   tExtra = new TExtraInfo();
   tExtra.setKey("dcau");
   tExtra.setValue(""+dcau);
   vec.addElement(tExtra);
   if(_debug) {
     util.printMessage("SRM-CLIENT: DCAU="+dcau, silent);
     util.printMessage("SRM-CLIENT: StorageInfo="+storageInfo, silent);
   }
   inputVec.addElement("DCAU="+dcau);
   inputVec.addElement("StorageInfo="+storageInfo);

   if(storageInfo.equals("true")) {

     tExtra = new TExtraInfo();
     tExtra.setKey("uid");
     tExtra.setValue("get");
     vec.addElement(tExtra);

     tExtra = new TExtraInfo();
     tExtra.setKey("pwd");
     tExtra.setValue(proxyString);
     vec.addElement(tExtra);
  }
  else if(storageInfo.startsWith("for")) {
     StringTokenizer stk = new StringTokenizer(storageInfo,",");
     while(stk.hasMoreTokens()) {
         String temp = stk.nextToken();
         int idx = temp.indexOf(":");
         if(idx != -1) {
           String key = temp.substring(0,idx);
           String value = temp.substring(idx+1);
           tExtra = new TExtraInfo();
           tExtra.setKey(key);
           tExtra.setValue(value);
           vec.addElement(tExtra);
         }
         else {
           String msg = "Given storage info is not in the correct format " +
            "Please use the format for:source,login:uid,passwd:pwd" +
            " Given storage info is " + storageInfo;
           throw new Exception(msg);
         }
      }
  }

  TExtraInfo[] a_tExtra = new TExtraInfo[vec.size()];
  for(int i = 0; i < vec.size(); i++) {
    a_tExtra[i] = (TExtraInfo) vec.elementAt(i);
  }
  storageInfoArray.setExtraInfoArray(a_tExtra);
  r.setStorageSystemInfo(storageInfoArray);

  ArrayOfTGetFileRequest arrayOfFileRequest = new ArrayOfTGetFileRequest();
  arrayOfFileRequest.setRequestArray(fileReqList);
  r.setArrayOfFileRequests(arrayOfFileRequest);

  util.printEventLog(_theLogger,"Input parameters for SrmBringOnlineRequest",inputVec,silent);
  util.printMessage("\nSRM-CLIENT: " + new Date() + 
		" Calling SrmBringOnlineRequest now ...", silent);

  inputVec = new Vector();
  inputVec.addElement("TimeStamp="+new Date());
  util.printEventLog(_theLogger,"Sending SrmBringOnlineRequest now",inputVec,silent);

  try {
     SrmBringOnlineResponse result = _srm.srmBringOnline(r);
     util.printMessage("\nSRM-CLIENT: " + new Date() + 
		" Returning result from the srm server ...", silent);
     return result;
  }catch(Exception e) {
      inputVec = new Vector();
      inputVec.addElement(e.getMessage());
      util.printEventLog(_theLogger,"Exception",inputVec,silent);
      util.printMessage("SRM-CLIENT: " + e.getMessage(),silent);
      util.printStackTrace(e,_theLogger);
      throw e;
  }
}

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// doStatus
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

SrmStatusOfBringOnlineRequestResponse doStatus (String requestId, String[] surl) 
	throws Exception {

   util.printMessage("SRM-CLIENT: "+ 
        " Calling Status at " + new Date(),silent);

   inputVec = new Vector();
   inputVec.addElement("RID="+requestId);
   util.printEventLog(_theLogger,"Input parameters for SrmStatusOfBringOnlineRequest",inputVec,silent);

   if(_debug) {
     util.printMessage(
		"SRM-CLIENT: ::::::::::::: Input parameters for SrmStatusOfBringOnlineRequest ::::", silent);
     util.printMessage("SRM-CLIENT: RID="+requestId,silent);
   }
 
   if(requestId == null || requestId.equals("")) {
     inputVec = new Vector();
     inputVec.addElement("SRM return null request token");
     util.printMessage("\nSRM-CLIENT: " +
            "SRM return null request token", silent);
     util.printEventLog(_theLogger,"SrmStatusOfBringOnlineResponse",inputVec,silent);
     return null;
   }

   SrmStatusOfBringOnlineRequestRequest r = new SrmStatusOfBringOnlineRequestRequest();
   
   int size = surl.length;

   URI[] tsurl = new URI[size];
   for(int i = 0; i < size; i++) {
    URI temp = new URI(surl[i]);
    tsurl[i] = temp;
   }

   if (size > 0) {
      r.setArrayOfSourceSURLs(SRMUtilClient.convertToArrayOfAnyURI(tsurl));
   }

   r.setRequestToken(requestId);
   inputVec = new Vector ();
   inputVec.addElement("Status Calling at " + new Date());
   util.printEventLog(_theLogger,"SrmStatusOfBringOnlineRequest",inputVec,silent);


   SrmStatusOfBringOnlineRequestResponse result = _srm.srmStatusOfBringOnlineRequest(r);

   if(result == null) {
     inputVec = new Vector();
     inputVec.addElement("SRM return null result");
     util.printMessage("\nSRM-CLIENT: " +
            "SRM return null result", silent);
     util.printEventLog(_theLogger,"SrmStatusOfBringOnlineResponse",inputVec,silent);
     return result;
   }

   if(result.getReturnStatus() == null) {
     inputVec = new Vector();
     inputVec.addElement("SRM return null return status");
     util.printMessage("\nSRM-CLIENT: " +
            "SRM return null return status", silent);
     util.printEventLog(_theLogger,"SrmStatusOfBringOnlineResponse",inputVec,silent);
     return result;
   }

   String explanation = result.getReturnStatus().getExplanation();

   if(_debug) {
     util.printMessage("\nSRM-CLIENT: ### Output from SRM ###",silent);
   }

   inputVec = new Vector();
   inputVec.addElement("Result Status from SRM (srmStatusOfBringOnlineRequest)=" +
        result.getReturnStatus().getStatusCode());
   if(_debug) {
     util.printMessage("Status=" + result.getReturnStatus().getStatusCode(), silent);
   } 

   if(explanation != null) {
       if(_debug) {
         util.printMessage("\tSRM-CLIENT: " +
            "Explanation=" + explanation, silent);
       }
       inputVec.addElement("\tCheck Status Explanation=" + explanation);
   }
   util.printEventLog(_theLogger,"SrmStatusOfBringOnlineResponse",inputVec,silent);

   TBringOnlineRequestFileStatus[] fileStatus = null;

   if(result.getArrayOfFileStatuses() != null) {
       fileStatus = result.getArrayOfFileStatuses().getStatusArray();
       if(fileStatus != null) {
         for(int i = 0; i < fileStatus.length; i++) {
           TBringOnlineRequestFileStatus status = fileStatus[i];
           if(_debug) {
            if(status != null) {
              util.printMessage("\tSURL="+status.getSourceSURL(),silent);
              if(status.getStatus() != null) {
                util.printMessage("\tStatus="+status.getStatus().getStatusCode(),silent);
                util.printMessage("\tExplanation="+
			  	   status.getStatus().getExplanation(),silent);
              }
              if(status.getFileSize() != null) {    
                util.printMessage("\tFileSize="+status.getFileSize(),silent);
              }
              if(status.getRemainingPinTime() != null) {    
                util.printMessage("\tRemainingPinTime="+status.getRemainingPinTime(),silent);
              }
              if(status.getEstimatedWaitTime() != null) {    
                util.printMessage("\tEstimatedWaitTime="+status.getEstimatedWaitTime(),silent);
              }
            }
           }
          }
       }
   }

   return result;
}



//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// printBringOnlineResult
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

static void printBringOnlineResult (SrmBringOnlineResponse response) throws Exception {

    if(_debug) {
      util.printMessage("\nSRM-CLIENT: ####### Output from SRM #####",silent);
    }

    if (response == null) {
      inputVec = new Vector();
      inputVec.addElement("printBringOnlineResult:SRM returned null response");
      util.printEventLog(_theLogger,"SrmBringOnlineResponse",inputVec,silent);
      util.printMessage("SRM-CLIENT: " +
            "printBringOnlineResult:SRM returned null response", silent);
    }

    if(response.getRequestToken() != null) {
      inputVec = new Vector();
      inputVec.addElement("request.token="+response.getRequestToken());
      util.printMessage("request.token="+ response.getRequestToken(),silent);
    }
    else {
      inputVec = new Vector();
      inputVec.addElement("request.token=null");
      util.printMessage("request.token=null",silent);
    }

    if(response.getReturnStatus() == null) {
       inputVec = new Vector();
       inputVec.addElement("printBringOnlineResult:SRM returned null return status for the response");
       util.printMessage(
		"SRM-CLIENT: printBringOnlineResult:SRM returned null return status for the response", silent);
       util.printEventLog(_theLogger,"SrmBringOnlineResponse",inputVec,silent);
       return;
    }

    inputVec.addElement("response.status="+response.getReturnStatus().getStatusCode());
    inputVec.addElement("response.explanation="+response.getReturnStatus().getExplanation());

    util.printMessage("status="+
      response.getReturnStatus().getStatusCode(),silent);
    util.printMessage("explanation="+
      response.getReturnStatus().getExplanation(),silent);

    if(response.getRemainingTotalRequestTime() != null) {
        inputVec.addElement("request.remainingTotalRequestTime="+
            response.getRemainingTotalRequestTime());
        if(_debug) {
          util.printMessage("request.remainingTotalRequestTime=" +
             response.getRemainingTotalRequestTime(),silent);
        }
    }
    else {
        inputVec.addElement("request.remainingTotalRequestTime=null");
        if(_debug) {
          util.printMessage("request.remainingTotalRequestTime=" +
             null,silent);
        }
    }

    if(response.getArrayOfFileStatuses() == null) {
       inputVec.addElement("printBringOnlineResult:SRM returns null getArrayOfFileStatuses");
       util.printEventLog(_theLogger,"SrmBringOnlineResponse",inputVec,silent);
       util.printMessage("SRM-CLIENT: " + 
			"printBringOnlineResult:SRM returns null getArrayOfFileStatuses ", silent);
       return;
    }

    if(response.getArrayOfFileStatuses().getStatusArray() == null) {
       inputVec.addElement("printBringOnlineResult:SRM returns null getStatusArray");
       util.printEventLog(_theLogger,"SrmBringOnlineResponse",inputVec,silent);
       util.printMessage("SRM-CLIENT: " +
            "printBringOnlineResult:SRM returns null getStatusArray", silent);
       return;
    }

    int size = response.getArrayOfFileStatuses().getStatusArray().length;

    util.printEventLog(_theLogger,"SrmBringOnlineResponse",inputVec,silent);

     for (int i=0;  i < size; i++) {
       TBringOnlineRequestFileStatus fileStatus =
         response.getArrayOfFileStatuses().getStatusArray(i);
       if(fileStatus != null) {
         inputVec = new Vector();
         if(fileStatus.getStatus() != null) {
           inputVec.addElement("Status="+fileStatus.getStatus().getStatusCode());
           inputVec.addElement("Explanation="+fileStatus.getStatus().getExplanation());
           util.printMessage("\tStatus="+
            fileStatus.getStatus().getStatusCode(),silent);
           util.printMessage("\tExplanation="+
            fileStatus.getStatus().getExplanation(),silent);
         }
         else {
           inputVec.addElement("Status=null");
           inputVec.addElement("Explanation=null");
           util.printMessage("\tStatus=null",silent);
           util.printMessage("\tExplanation=null",silent);
         }
         if(fileStatus.getSourceSURL() != null) {
           util.printMessage("\tSURL="+
              fileStatus.getSourceSURL().toString(),silent);
           inputVec.addElement("SURL="+fileStatus.getSourceSURL().toString());
         }
         util.printEventLog(_theLogger,"SrmBringOnlineResponse",inputVec,silent);
       }
   }

   if(_debug) {
     util.printMessage("...........................................",silent);
   }
}


}

